package com.subd.subd.Services;

import com.subd.subd.Exceptions.CarNotFoundException;
import com.subd.subd.Exceptions.DriverNotFoundException;
import com.subd.subd.Models.Car;
import com.subd.subd.Models.Driver;
import com.subd.subd.Repositories.CarRepository;
import com.subd.subd.Repositories.DriverRepository;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Optional;

@Service
public class CarService {
    private final CarRepository carRepository;
    private final DriverRepository driverRepository;
    public CarService(CarRepository carRepository, DriverRepository driverRepository) {
        this.carRepository = carRepository;
        this.driverRepository = driverRepository;
    }
    @Transactional
    public Car addCar(String gosNumber, String vin, Long driverId) {
        final Driver driver = driverRepository.findById(driverId)
                .orElseThrow(() -> new DriverNotFoundException(driverId));
        final Car car = new Car(gosNumber, vin, driver);
        return carRepository.save(car);
    }
    @Transactional(readOnly = true)
    public Car findCar(Long id) {
        final Optional<Car> car = carRepository.findById(id);
        return car.orElseThrow(() -> new CarNotFoundException(id));
    }
    @Transactional(readOnly = true)
    public List<Car> findAllCars() {
        return carRepository.findAll();
    }
    @Transactional
    public Car updateCar(Long id, String gosNumber, String vin, Long driverId) {
        final Car currentCar = findCar(id);
        if (gosNumber != null) {
            currentCar.setGosNumber(gosNumber);
        }
        if (vin != null) {
            currentCar.setVin(vin);
        }
        if (driverId != null) {
            final Driver driver = driverRepository.findById(driverId)
                            .orElseThrow(() -> new DriverNotFoundException(driverId));
            currentCar.setDriver(driver);
        }
        return carRepository.save(currentCar);
    }
    @Transactional
    public Driver getDriver(Long carId) {
        Car currentCar = carRepository.findById(carId)
                .orElseThrow(() -> new CarNotFoundException(carId));
        return currentCar.getDriver();
    }
    @Transactional
    public void setDriver(Long carId, Long driverId) {
        Car currentCar = carRepository.findById(carId)
                .orElseThrow(() -> new CarNotFoundException(carId));
        Driver driver = driverRepository.findById(driverId)
                .orElseThrow(() -> new DriverNotFoundException(driverId));
        currentCar.setDriver(driver);
    }
    @Transactional
    public Car deleteCar(Long id) {
        final Car currentCar = findCar(id);
        carRepository.delete(currentCar);
        return currentCar;
    }
    @Transactional
    public void deleteAllCars() {
        carRepository.deleteAll();
    }
}
