package com.subd.subd.Services;

import com.subd.subd.Exceptions.ClientNotFoundException;
import com.subd.subd.Models.Client;
import com.subd.subd.Repositories.ClientRepository;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Optional;

@Service
public class ClientService {
    private final ClientRepository clientRepository;
    public ClientService(ClientRepository clientRepository) {
        this.clientRepository = clientRepository;
    }
    @Transactional
    public Client addClient(String name, String phone, String email) {
        final Client client = new Client(name, phone, email);
        return clientRepository.save(client);
    }
    @Transactional(readOnly = true)
    public Client findClient(Long id) {
        final Optional<Client> client = clientRepository.findById(id);
        return client.orElseThrow(() -> new ClientNotFoundException(id));
    }
    @Transactional(readOnly = true)
    public List<Client> findAllClients() {
        return clientRepository.findAll();
    }
    @Transactional
    public Client updateClient(Long id, String name, String phone, String email) {
        final Client currentClient = findClient(id);
        if (name != null) {
            currentClient.setName(name);
        }
        if (phone != null) {
            currentClient.setPhone(phone);
        }
        if (email != null) {
            currentClient.setEmail(email);
        }
        return clientRepository.save(currentClient);
    }
    @Transactional
    public Client deleteClient(Long id) {
        final Client currentClient = findClient(id);
        clientRepository.delete(currentClient);
        return currentClient;
    }
    @Transactional
    public void deleteAllClients() {
        clientRepository.deleteAll();
    }
}
