package com.subd.subd.Services;

import com.subd.subd.Exceptions.DriverNotFoundException;
import com.subd.subd.Models.Driver;
import com.subd.subd.Repositories.DriverRepository;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;
import java.util.Optional;

@Service
public class DriverService {
    private final DriverRepository driverRepository;
    public DriverService(DriverRepository driverRepository) {
        this.driverRepository = driverRepository;
    }
    @Transactional
    public Driver addDriver(String name, Date birthday, String phone, String email) {
        final Driver driver = new Driver(name, birthday, phone, email);
        return driverRepository.save(driver);
    }
    @Transactional(readOnly = true)
    public Driver findDriver(Long id) {
        final Optional<Driver> driver = driverRepository.findById(id);
        return driver.orElseThrow(() -> new DriverNotFoundException(id));
    }
    @Transactional(readOnly = true)
    public List<Driver> findAllDrivers() {
        return driverRepository.findAll();
    }
    @Transactional
    public Driver updateDriver(Long id, String name, Date birthday, String phone, String email) {
        final Driver currentDriver = findDriver(id);
        if (name != null) {
            currentDriver.setName(name);
        }
        if (birthday != null) {
            currentDriver.setBirthday(birthday);
        }
        if (phone != null) {
            currentDriver.setPhone(phone);
        }
        if (email != null) {
            currentDriver.setEmail(email);
        }
        return driverRepository.save(currentDriver);
    }
    @Transactional
    public Driver deleteDriver(Long id) {
        final Driver currentDriver = findDriver(id);
        driverRepository.delete(currentDriver);
        return currentDriver;
    }
    @Transactional
    public void deleteAllDrivers() {
        driverRepository.deleteAll();
    }
}
