import ToolBar from '../components/ToolBar.vue';
import DataTable from '../components/DataTable.vue';
import Modal from '../components/Modal.vue';
import DataService from '../services/DataService';

const CatalogMixin = {
    components: {
        ToolBar, DataTable, Modal
    },
    data() {
        return {
            getAllUrl: undefined,
            dataUrl: undefined,
            transformer: undefined,
            headers: [],
            items: [],
            selectedItems: [],
            modal: {
                header: undefined,
                confirm: undefined,
            },
            modalShow: false,
            data: undefined,
            isEdit: false
        }
    },
    created() {
        this.loadItems();
    },
    methods: {
        loadItems() {
            this.getItems();
            this.data = this.transformer();
        },
        getItems() {
            DataService.readAll(this.getAllUrl, this.transformer)
                .then(data => {
                    this.items = data;
                });
        },
        showAddModal() {
            this.isEdit = false;
            this.data = this.transformer();
            this.modal.header = 'Добавление элемента';
            this.modal.confirm = 'Добавить';
            this.modalShow = true;
        },
        showEditModal() {
            if (this.selectedItems.length === 0) {
                return;
            }
            this.showEditModalDblClick(this.selectedItems[0]);
        },
        showEditModalDblClick(editId) {
            DataService.read(this.dataUrl + "/" + editId, this.transformer)
                .then(data => {
                    this.data = data;
                    this.isEdit = true;
                    this.modal.header = 'Редактирование элемента';
                    this.modal.confirm = 'Сохранить';
                    this.modalShow = true;
                });
        },
        saveItem() {
            if (!this.isEdit) {
                DataService.create(this.dataUrl, this.data)
                    .then(() => {
                        this.getItems();
                    });
            } else {
                DataService.update(this.dataUrl + "/" + this.data.id, this.data)
                    .then(() => {
                        this.getItems();
                    });
            }
        },
        removeSelectedItems() {
            if (this.selectedItems.length === 0) {
                return;
            }
            if (confirm('Удалить выбранные элементы?')) {
                const promises = [];
                const self = this;
                this.selectedItems.forEach(item => {
                    promises.push(DataService.delete(this.dataUrl + item));
                });
                Promise.all(promises).then((results) => {
                    results.forEach(function (id) {
                        const index = self.selectedItems.indexOf(id);
                        if (index === - 1) {
                            return;
                        }
                        self.selectedItems.splice(index, 1);
                    });
                    this.getItems();
                });
            }
        }
    }
}

export default CatalogMixin;